@@ -81,16 +81,21 @@ class SessionMiddleware:
 
         async def send_wrapper(message: Message, **kwargs) -> None:
             if message["type"] == "http.response.start":
-                if scope["session"]:
 
-                    session_key = str(uuid4())
+                data = connection.cookies.get(self.cookie_name)
+                data = data.encode("utf-8") if data else None
+
+                session_key = (
+                    json.loads(b64decode(data)).get(self._cookie_session_id_field)
+                    if data
+                    else str(uuid4())
+                )
+
+                if scope["session"]:
 
                     if self.backend_type == BackendType.cookie:
                         cookie_data = scope["session"]
                     else:
-                        self.session_backend.set(
-                            session_key, scope["session"], self.max_age
-                        )
                         cookie_data = {self._cookie_session_id_field: session_key}
 
                     data = b64encode(json.dumps(cookie_data).encode("utf-8"))
@@ -103,10 +108,6 @@ class SessionMiddleware:
                 elif not initial_session_was_empty:
 
                     if self.backend_type != BackendType.cookie:
-                        data = connection.cookies[self.cookie_name].encode("utf-8")
-                        session_key = json.loads(b64decode(data)).get(
-                            self._cookie_session_id_field
-                        )
                         self.session_backend.delete(session_key)
 
                     headers = MutableHeaders(scope=message)
